/*****************************************************************************
   mixer.c - mixer code
   Copyright (C) 2000 by Jaroslav Kysela <perex@suse.cz>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include "envy24control.h"

#define toggle_set(widget, state) \
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), state);

static int is_active(GtkWidget *widget)
{
	return GTK_TOGGLE_BUTTON(widget)->active ? 1 : 0;
}

void mixer_update_stream(int stream, int vol_flag, int sw_flag)
{
	snd_mixer_element_t vol, sw;
	int err;
	
	if (vol_flag) {
		memset(&vol, 0, sizeof(vol));
		strcpy(vol.eid.name, stream <= 10 ? "Multi Playback Volume" : "Multi Capture Volume");
		vol.eid.index = (stream - 1) % 10;
		vol.eid.type = SND_MIXER_ETYPE_VOLUME1;
		if ((err = snd_mixer_element_build(card_mixer, &vol)) < 0)
			g_print("Unable to read multi playback volume: %s\n", snd_strerror(err));
		gtk_adjustment_set_value(GTK_ADJUSTMENT(mixer_adj[stream-1][0]), 96 - vol.data.volume1.pvoices[0]);
		gtk_adjustment_set_value(GTK_ADJUSTMENT(mixer_adj[stream-1][1]), 96 - vol.data.volume1.pvoices[1]);
		if (vol.data.volume1.pvoices[0] != vol.data.volume1.pvoices[1])
			toggle_set(mixer_stereo_toggle[stream-1], FALSE);
		snd_mixer_element_free(&vol);
	}
	if (sw_flag) {
		memset(&sw, 0, sizeof(sw));
		strcpy(sw.eid.name, stream <= 10 ? "Multi Playback Switch" : "Multi Capture Switch");
		sw.eid.index = (stream - 1) % 10;
		sw.eid.type = SND_MIXER_ETYPE_SWITCH1;
		if ((err = snd_mixer_element_build(card_mixer, &sw)) < 0)
			g_print("Unable to read multi playback switch: %s\n", snd_strerror(err));
		toggle_set(mixer_solo_toggle[stream-1][0], snd_mixer_get_bit(sw.data.switch1.psw, 0) ? TRUE : FALSE);
		toggle_set(mixer_solo_toggle[stream-1][1], snd_mixer_get_bit(sw.data.switch1.psw, 1) ? TRUE : FALSE);
		toggle_set(mixer_mute_toggle[stream-1][0], !snd_mixer_get_bit(sw.data.switch1.psw, 0) ? TRUE : FALSE);
		toggle_set(mixer_mute_toggle[stream-1][1], !snd_mixer_get_bit(sw.data.switch1.psw, 1) ? TRUE : FALSE);
		if (snd_mixer_get_bit(sw.data.switch1.psw, 0) != snd_mixer_get_bit(sw.data.switch1.psw, 1))
			toggle_set(mixer_stereo_toggle[stream-1], FALSE);
		snd_mixer_element_free(&sw);
	}
}

static void set_switch1(int stream, int left, int right)
{
	snd_mixer_element_t sw;
	int err;
	
	memset(&sw, 0, sizeof(sw));
	strcpy(sw.eid.name, stream <= 10 ? "Multi Playback Switch" : "Multi Capture Switch");
	sw.eid.index = (stream - 1) % 10;
	sw.eid.type = SND_MIXER_ETYPE_SWITCH1;
	if ((err = snd_mixer_element_build(card_mixer, &sw)) < 0)
		g_print("Unable to read multi switch: %s\n", snd_strerror(err));
	if (left >= 0)
		snd_mixer_set_bit(sw.data.switch1.psw, 0, left);
	if (right >= 0)
		snd_mixer_set_bit(sw.data.switch1.psw, 1, right);
	if ((err = snd_mixer_element_write(card_mixer, &sw)) < 0 && err != -EBUSY)
		g_print("Unable to write multi switch: %s\n", snd_strerror(err));
	snd_mixer_element_free(&sw);
}

void mixer_toggled_solo(GtkWidget *togglebutton, gpointer data)
{
	int stream = (int)data >> 16;
	int button = (int)data & 1;
	int stereo = is_active(mixer_stereo_toggle[stream-1]) ? 1 : 0;
	int vol[2] = { -1, -1 };

	if (is_active(togglebutton)) {
		if (is_active(mixer_mute_toggle[stream-1][button]))
			toggle_set(mixer_mute_toggle[stream-1][button], FALSE);
		vol[button] = 1;
		if (stereo) {
			if (!is_active(mixer_solo_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_solo_toggle[stream-1][button ^ 1], TRUE);
			if (is_active(mixer_mute_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_mute_toggle[stream-1][button ^ 1], FALSE);
			vol[button ^ 1] = 1;
		}
	} else {
		if (!is_active(mixer_mute_toggle[stream-1][button]))
			toggle_set(mixer_mute_toggle[stream-1][button], TRUE);
		vol[button] = 0;
		if (stereo) {
			if (is_active(mixer_solo_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_solo_toggle[stream-1][button ^ 1], FALSE);
			if (!is_active(mixer_mute_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_mute_toggle[stream-1][button ^ 1], TRUE);
			vol[button ^ 1] = 0;
		}
	}
	set_switch1(stream, vol[0], vol[1]);
}

void mixer_toggled_mute(GtkWidget *togglebutton, gpointer data)
{
	int stream = (int)data >> 16;
	int button = (int)data & 1;
	int stereo = is_active(mixer_stereo_toggle[stream-1]) ? 1 : 0;
	int vol[2] = { -1, -1 };

	if (is_active(togglebutton)) {
		if (is_active(mixer_solo_toggle[stream-1][button]))
			toggle_set(mixer_solo_toggle[stream-1][button], FALSE);
		vol[button] = 0;
		if (stereo) {
			if (!is_active(mixer_mute_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_mute_toggle[stream-1][button ^ 1], TRUE);
			if (is_active(mixer_solo_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_solo_toggle[stream-1][button ^ 1], FALSE);
			vol[button ^ 1] = 0;
		}
	} else {
		if (!is_active(mixer_solo_toggle[stream-1][button]))
			toggle_set(mixer_solo_toggle[stream-1][button], TRUE);
		vol[button] = 1;
		if (stereo) {
			if (is_active(mixer_mute_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_mute_toggle[stream-1][button ^ 1], FALSE);
			if (!is_active(mixer_solo_toggle[stream-1][button ^ 1]))
				toggle_set(mixer_solo_toggle[stream-1][button ^ 1], TRUE);
			vol[button ^ 1] = 1;
		}
	}
	set_switch1(stream, vol[0], vol[1]);
}

static void set_volume1(int stream, int left, int right)
{
	snd_mixer_element_t vol;
	int err;
	
	memset(&vol, 0, sizeof(vol));
	strcpy(vol.eid.name, stream <= 10 ? "Multi Playback Volume" : "Multi Capture Volume");
	vol.eid.index = (stream - 1) % 10;
	vol.eid.type = SND_MIXER_ETYPE_VOLUME1;
	if ((err = snd_mixer_element_build(card_mixer, &vol)) < 0)
		g_print("Unable to read multi volume: %s\n", snd_strerror(err));
	if (left >= 0)
		vol.data.volume1.pvoices[0] = left;
	if (right >= 0)
		vol.data.volume1.pvoices[1] = right;
	if ((err = snd_mixer_element_write(card_mixer, &vol)) < 0 && err != -EBUSY)
		g_print("Unable to write multi volume: %s\n", snd_strerror(err));
	snd_mixer_element_free(&vol);
}

void mixer_adjust(GtkAdjustment *adj, gpointer data)
{
	int stream = (int)data >> 16;
	int button = (int)data & 1;
	int stereo = is_active(mixer_stereo_toggle[stream-1]) ? 1 : 0;
	int vol[2] = { -1, -1 };
	
	vol[button] = 96 - adj->value;
	if (stereo) {
		gtk_adjustment_set_value(GTK_ADJUSTMENT(mixer_adj[stream-1][button ^ 1]), adj->value);
		vol[button ^ 1] = 96 - adj->value;
	}
	set_volume1(stream, vol[0], vol[1]);
}

void mixer_postinit(void)
{
	int stream;

	for (stream = 1; stream <= 20; stream++)
		mixer_update_stream(stream, 1, 1);
}
